#include <errno.h>
#include <iostream>
using namespace std;
#include "bufdio.hh"
#include "logger.hh"

bool io_buffer::in_get_block(virtual_buffer<char>& buf, const size_t len)
{
	if (inbuf.count_elements_left() < len) {
		want_read = len - inbuf.count_elements_left();
		return false;
	}
	virtual_buffer<char> nb(inbuf, len);
	buf = nb;
	inbuf.advance(len);
	return true;
}

bool io_buffer::in_get_min(virtual_buffer<char>& buf, const size_t minlen)
{
	if (inbuf.count_elements_left() < minlen) {
		want_read = minlen - inbuf.count_elements_left();
		return false;
	}
	buf = inbuf;
	inbuf.advance(buf.count_elements_left());
	return true;
}

void buffered_io::wait_for_write(bool wait)
{
	del(*this);
	set(wait ? EV_READ|EV_WRITE|EV_PERSIST : EV_READ|EV_PERSIST);
	add(*this);
}

void buffered_io::poll_read()
{
	char *ptr = inbuf.append_space(BUFBLOCK);
	ssize_t n = ::recv(fd, ptr, BUFBLOCK, 0);

	inbuf.shrink(n < 0 ? BUFBLOCK : BUFBLOCK - n);

	if (n <= 0) {
		logger	<< timestamp << " recv error on [" << fd << "]: "
			<< errno_str(n ? errno : 0) << endl;
		if (!n)
			poll_error(0);
		else if (errno != EINTR && errno != EAGAIN)
			poll_error(errno);
		return;
	}

	if (DEBUG(PACKETDATA) || DEBUG(PACKETFLOW)) {
		virtual_buffer<char> data(ptr, n);
		logger	<< timestamp << " data on [" << fd << "]; want " << want_read << " got "
			<< inbuf.count_elements_left() << "; ";
		if (DEBUG(PACKETDATA))
			logger << data;
		else	logger << n << " this time" << endl;
	}

	if (inbuf.count_elements_left() >= want_read)
		want_read = 0;

	if (!want_read)
		buf_read();
}

void buffered_io::poll_write()
{
	if (!outbuf.count_elements_left()) {
		logger << timestamp << " **BUG** poll_write() with empty queue on [" << fd << ']' << endl;
		finish_write();
		return;
	}

	ssize_t n = ::send(fd, outbuf.get_head_ptr(), outbuf.count_elements_left(), MSG_DONTWAIT|MSG_NOSIGNAL);

	if (n <= 0) {
		logger	<< timestamp << " send error on [" << fd << "]: "
			<< errno_str(n ? errno : 0) << endl;
		if (!n)
			poll_error(0);
		else if (errno != EINTR && errno != EAGAIN)
			poll_error(errno);
		return;
	}

	if (DEBUG(PACKETFLOW)|DEBUG(PACKETDATA)) {
		logger	<< timestamp << " data sent on [" << fd << "]: " << n 
			<< '/' << outbuf.count_elements_left();
		if (DEBUG(PACKETDATA)) {
			virtual_buffer<char> tmp(outbuf, n);
			logger	<< ' ' << tmp;
		} else
			logger	<< endl;
	}

	outbuf.advance(n);
	outbuf.clean();
	finish_write();
}


