#ifndef BUFDIO_HH
#define BUFDIO_HH

#include "ioclass.hh"
#include "buffer.hh"
#include <sys/types.h>

class io_buffer
{
public:
	buffer<char> inbuf;
protected:
	buffer<char> outbuf;
	size_t want_read;
	bool waiting_for_write;

	virtual void wait_for_write(bool wait) = 0;

	void start_write() {
		if (!outbuf.is_empty() && !waiting_for_write)
			wait_for_write(waiting_for_write = true);
	}
	void finish_write() {
		if (waiting_for_write && outbuf.is_empty())
			wait_for_write(waiting_for_write = false);
	}

	virtual void buf_read() = 0;
public:
	io_buffer() : want_read(0), waiting_for_write(false) {}
	virtual ~io_buffer() {}

	void cleanbufs() { inbuf.clean(); outbuf.clean(); }

	void out_append(const char *const data, const size_t len)
		{ outbuf.append(data, len); start_write(); }
	void out_append(const virtual_buffer<char>& buf)
		{ outbuf.append(buf); start_write(); }
	void out_append_and_move(virtual_buffer<char>& buf, const size_t copylen, const size_t skiplen = 0)
		{ outbuf.append_and_move(buf, copylen, skiplen); start_write(); }

	bool in_get_block(virtual_buffer<char>& buf, const size_t len);
	bool in_get_min(virtual_buffer<char>& buf, const size_t minlen);

	bool readline(virtual_buffer<char>& buf, const virtual_buffer<char>& rl_ending)
		{ return inbuf.readline(buf, rl_ending); }
};

class buffered_io : public io_socket, public io_buffer
{
protected:
	void wait_for_write(bool wait);
	void poll_read();
	void poll_write();
public:
	buffered_io(int _fd, bool _own = true) : io_socket(_fd, _own), io_buffer() {}
};

#endif /* !BUFDIO_HH */
