#ifndef EVENTWRAP_HH
#define EVENTWRAP_HH

/* libevent - C++ wrapper */

#include <sys/time.h>
namespace __ev {
typedef unsigned char u_char;
#include <event.h>
};

extern "C" void dump_backtrace(const void *self, const char *why, int skip);

class timeout
{
	friend class event;
private:
	struct timeval tv;
public:
	timeout(int sec, int usec) { tv.tv_sec = sec; tv.tv_usec = usec; }
};

class event
{
protected:
	virtual void handle(int fd, int evflags) = 0;
private:
	struct __ev::event ev;

	static void call_handler(int fd, short int evflags, void *arg);
public:
	void dump_me(const char *why) const;

	event(int fd, int evflags) {
		__ev::event_set(&ev, fd, evflags, call_handler, static_cast<void *>(this));
	}
	virtual ~event();

	void set(int evflags) {
		__ev::event_set(&ev, ev.ev_fd, evflags, call_handler, static_cast<void *>(this));
	}
	void set(int fd, int evflags) {
		__ev::event_set(&ev, fd, evflags, call_handler, static_cast<void *>(this));
	}

	static void add(event& ev) {
		__ev::event_add(&ev.ev, 0);
	}
	static void add(event& ev, const timeout& to) {
		__ev::event_add(&ev.ev, const_cast<struct timeval *>(&to.tv));
	}
	static void del(event& ev) {
		__ev::event_del(&ev.ev);
	}
	static void loop() {
		__ev::event_dispatch();
	}

	void add() { add(*this); }
	void add(const timeout& to) { add(*this, to); }
	void del() { del(*this); }
};

class evtimer : public event
{
public:
	evtimer() : event(-1, 0) {}
};

class evsignal : public event
{
public:
	evsignal(int sig) : event(sig, EV_SIGNAL|EV_PERSIST) {}
};

#endif /* !EVENTWRAP_HH */
