#include "logger.hh"
#include "bnetproto.hh"
#include <sys/time.h>
#include <string.h>
#include <iostream>
#include <iomanip>
using namespace std;

unsigned int debug = ~(DEBUG_PACKETDATA|DEBUG_PACKETFLOW|DEBUG_DATASUBST);

ostream& timestamp(ostream& os)
{
	struct timeval tv;
	gettimeofday(&tv, NULL);
	char fill = os.fill();
	return os << tv.tv_sec << '.' << setw(6) << setfill('0') << tv.tv_usec << setfill(fill);
}

logger_stream::logger_stream(const char *fn)
	: ofstream(fn, ios::out|ios::app)
{
}

ostream& operator<<(ostream& os, const errno_str& estr)
{
	return os << strerror(estr.err) << " (" << estr.err << ')';
}

ostream& operator<<(ostream& os, const io_socket& io)
{
	socket_address sa;

	try {
		io.get_my_address(sa);
		os << sa;
	} catch(int) {
		os << "[unknown]";
	}

	os << " <- ";

	try {
		io.get_peer_address(sa);
		os << sa;
	} catch(int) {
		os << "[unknown]";
	}

	return os;
}

ostream& operator<<(ostream& os, const udp_mx_peer& udppeer)
{
	return os << udppeer.remote;
}

template <class T>
void dump_hex1(ostream& os, virtual_buffer<T>& buf, char *)
{
	os << setw(sizeof(T) * 2) << *buf.get_head_ptr();
}

template <>
void dump_hex1(ostream& os, virtual_buffer<char>& buf, char *charlist)
{
	char x = *buf.get_head_ptr();
	if (x < ' ' || x >= 127) x = '.';
	*charlist = x;
	os << setw(sizeof(char) * 2) << (int)(unsigned char)*buf.get_head_ptr();
}

template <class T>
ostream& operator<<(ostream& os, const virtual_buffer<T>& _buf)
{
	virtual_buffer<T> buf(_buf);

	os << " buf[" << buf.count_elements_left() << "] = ";
	if (!buf.count_elements_left())
		return os << " {}" << endl;
	os << endl;

	const int w = 16 / sizeof(T);
	int l = w;
	char charlist[w + 1];
	charlist[w] = 0;

	os << setfill('0') << hex;
	while (buf.count_elements_left() > 0) {
		if (l == w)
			os << '\t';
		dump_hex1(os, buf, charlist + w - l);
		buf.advance(1);
		if (--l) {
			os << ' ';
			if (l == w >> 1)
				os << ' ';
		} else {
			os << "   " << charlist << endl;
			l = w;
		}
	}
	if (l != w) {
		charlist[w - l] = 0;
		while (l--) {
			os << "   ";
			if (l == w >> 1)
				os << ' ';
		}
		os << "  " << charlist << endl;
	}
	os << dec << setfill(' ');

	return os;
}

typedef ostream& (*f_out_char)(ostream&, const virtual_buffer<char>& buf);
static f_out_char lkdgjskl = operator<<;

std::ostream& operator<<(std::ostream& os, const bnet_packet_type_str& bptstr)
{
	switch (bptstr.type) {
		case BNET_PT_GAMEINFO:
			os << "gameinfo";
		break;
		case BNET_PT_HELLO:
			os << "hello";
		break;
		case BNET_PT_MESSAGE:
			os << "message";
		break;
		case BNET_PT_CHATCMD:
			os << "chat message/command";
		break;
		case BNET_PT_NEWGAME:
			os << "new game";
		break;
		default:
			os << "*unknown#" << hex << bptstr.type << dec << '*';
	}
	return os;
}

class debug_pkt_bitmap
{
protected:
	unsigned int bitmap[0x10000/8/sizeof(unsigned int)];	// 64kbit
public:
	debug_pkt_bitmap();
	bool operator[](unsigned t) const { return bitmap[t/8/sizeof(unsigned int)] & (1 << (t & (8*sizeof(unsigned int)-1))); }
	void set(unsigned t) { bitmap[t/8/sizeof(unsigned int)] |= (1 << (t & (8*sizeof(unsigned int)-1))); }
	void clear(unsigned t) { bitmap[t/8/sizeof(unsigned int)] &= ~(1 << (t & (8*sizeof(unsigned int)-1))); }
};

debug_pkt_bitmap::debug_pkt_bitmap()
{
	memset(bitmap, -1, sizeof(bitmap));
#if 1
	clear(0x0fff);	// message
	clear(0x33ff);
	clear(0x26ff);
	clear(0x25ff);
	clear(0x00ff);
	clear(0x2dff);
	clear(0x51ff);
	clear(0x15ff);
	clear(0x21ff);
	clear(0x0aff);
	clear(0x0bff);
	clear(0x0cff);
	clear(0x0eff);
	clear(0x29ff);

	clear(0x09ff);	// gameinfo
#endif
}

static debug_pkt_bitmap debug_pkt;

bool want_packet_dump(int packet_type)
{
	return debug_pkt[packet_type];
}

