#ifndef LOGGER_HH
#define LOGGER_HH

#include "ioclass.hh"
#include "udpmx.hh"
#include "buffer.hh"
#include <iostream>
#include <fstream>
#include <iomanip>

class errno_str
{
private:
	int err;
public:
	errno_str(int _err) : err(_err) {}

	friend std::ostream& operator<<(std::ostream& os, const errno_str& estr);
};

class bnet_packet_type_str
{
private:
	int type;
public:
	bnet_packet_type_str(int _type) : type(_type) {}

	friend std::ostream& operator<<(std::ostream& os, const bnet_packet_type_str& bptstr);
};

std::ostream& timestamp(std::ostream& os);
std::ostream& operator<<(std::ostream& os, const io_socket& io);
std::ostream& operator<<(std::ostream& os, const udp_mx_peer& udppeer);

template <class T>
std::ostream& operator<<(std::ostream& os, const virtual_buffer<T>& buf);

class logger_stream : public std::ofstream
{
public:
	logger_stream(const char *fn);
};

extern std::ostream& logger;

#define DEBUG_CONNS		0x00000001
#define DEBUG_PACKETDATA	0x00000002
#define DEBUG_PACKETFLOW	0x00000004
#define DEBUG_BNETTCPFLOW	0x00000008 
#define DEBUG_BNETTCPDATA	0x00000010 
#define DEBUG_BNETUDPFLOW	0x00000020 
#define DEBUG_BNETUDPDATA	0x00000040 
#define DEBUG_UDPMASKING	0x00000080
#define DEBUG_DATASUBST		0x00000100
#define DEBUG_DATAFORMAT	0x00000200

extern unsigned int debug;

#define DEBUG(l) ((debug & DEBUG_##l) != 0)

bool want_packet_dump(int packet_type);

#endif /* !LOGGER_HH */
