#ifndef SOCKADDR_HH
#define SOCKADDR_HH

#include "eventwrap.hh"
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <iostream>

class socket_address
{
public:
	struct sockaddr_storage sa;
	socklen_t salen;
	socklen_t salen_max() const { return sizeof(sa); }

	socket_address() : salen(sizeof(sa))
	{
		memset(&sa, 0, sizeof(sa));
		sa.ss_family = AF_UNSPEC;
	}

	const struct sockaddr *const_sockaddr() const { return (const struct sockaddr *)&sa; }
	struct sockaddr *sockaddr() { return (struct sockaddr *)&sa; }

	bool is_inet() const { return sa.ss_family == AF_INET; }
	const struct sockaddr_in *const_sockaddr_in() const { return (const struct sockaddr_in *)&sa; }
	struct sockaddr_in *sockaddr_in() { return (struct sockaddr_in *)&sa; }

	uint16_t get_inet_port() const { return ntohs(const_sockaddr_in()->sin_port); }
	void set_inet_port(uint16_t port) { sockaddr_in()->sin_port = htons(port); }

	uint32_t get_inet_addr() const { return ntohl(const_sockaddr_in()->sin_addr.s_addr); }
	void set_inet_addr(uint32_t addr) { sockaddr_in()->sin_addr.s_addr = htonl(addr); }
	void set_inet_addr(const char *addr) { sockaddr_in()->sin_addr.s_addr = ::inet_addr(addr); }

	void set_inet_address(uint32_t addr, uint16_t port) {
		sa.ss_family = AF_INET;
		salen = sizeof(struct sockaddr_in);
		sockaddr_in()->sin_addr.s_addr = htonl(addr);
		sockaddr_in()->sin_port = htons(port);
	}
	void set_inet_address(const char *addr, uint16_t port) {
		sa.ss_family = AF_INET;
		salen = sizeof(struct sockaddr_in);
		sockaddr_in()->sin_addr.s_addr = ::inet_addr(addr);
		sockaddr_in()->sin_port = htons(port);
	}

	socket_address(uint32_t ip, uint16_t port) : salen(sizeof(struct sockaddr_in))
	{
		set_inet_address(ip, port);
	}

	struct ipportpair
	{
		uint32_t ip;
		uint16_t port;

		ipportpair(uint32_t i, uint16_t p) : ip(i), port(p) {}
		bool operator==(const ipportpair& ipp) const
			{ return ip == ipp.ip && port == ipp.port; }

		friend std::ostream& operator<<(std::ostream& os, const ipportpair& ipp)
		{
			char buf[100];
			snprintf(buf, 100, "%08X:%d", ipp.ip, ipp.port);
			return os << buf;
		}
	};

	operator ipportpair() const { return ipportpair(get_inet_addr(), get_inet_port()); }

	socket_address(const ipportpair& ipport) : salen(sizeof(struct sockaddr_in))
	{
		memset(&sa, 0, sizeof(sa));
		sockaddr_in()->sin_family = AF_INET;
		sockaddr_in()->sin_addr.s_addr = htonl(ipport.ip);
		sockaddr_in()->sin_port = htons(ipport.port);
	}

protected:
	void print_inet_to(std::ostream& os) const {
		char buf[100];
//		snprintf(buf, 100, "%08X:%d (%d,%d)", get_inet_addr(), get_inet_port(), sa.ss_family, salen);
		snprintf(buf, 100, "%08X:%d", get_inet_addr(), get_inet_port());
		os << buf;
	}
public:
	std::ostream& print_to(std::ostream& os) const {
		if (is_inet())
			print_inet_to(os);
		else	os << "[unknown]";
		return os;
	}
};

inline std::ostream& operator<<(std::ostream& os, const socket_address& sa)
{
	return sa.print_to(os);
}

#endif /* !SOCKADDR_HH */
