#include <errno.h>
#include <iostream>
using namespace std;
#include "tcpforward.hh"
#include "logger.hh"

inline void tcp_forward::forwarding_side::start()
{
	event::set(EV_READ|EV_PERSIST);
	event::add(*this);
}

inline void tcp_forward::forwarding_side::stop()
{
	event::del(*this);
}

void tcp_forward::forwarding_side::buf_read()
{
	do {
		master->slave_read_callback(this);
	} while (want_read < inbuf.count_elements_left());
	inbuf.clean();
}

void tcp_forward::forwarding_side::poll_error(int err)
{
	master->slave_error_callback(this, err);
}

void tcp_forward::slave_read_callback(tcp_forward::forwarding_side *_slave)
{
	forwarding_side *other = slave[_slave == slave[0]];

	other->out_append(_slave->inbuf);
	_slave->inbuf.clear();
}

void tcp_forward::slave_error_callback(tcp_forward::forwarding_side *_slave, int err)
{
	delete this;
	throw err;
}

tcp_forward::~tcp_forward()
{
	if (slave[0]) {
		delete slave[0];
		delete slave[1];
	}
}

void tcp_forward::start_forward(int fds[2])
{
	slave[0] = new forwarding_side(this, fds[0]);
	if (!slave[0])
		throw int(ENOMEM);

	slave[1] = new forwarding_side(this, fds[1]);
	if (!slave[0]) {
		delete slave[0];
		slave[0] = NULL;
		throw int(ENOMEM);
	}

	slave[0]->start();
	slave[1]->start();

	if (DEBUG(CONNS))
		logger	<< timestamp << " new forward [" << fds[0] << ',' << fds[1]
			<< "] started" << endl;
}

void tcp_forward_listener::forward_creator::poll_connected()
{
	int fds[2] = { sibling_fd, fd };

	tcp_forward *fwd = master->create_forward();
	fwd->start_forward(fds);
	own = false;
	master->start_forwarding(this, fwd);
}

void tcp_forward_listener::forward_creator::poll_error(int err)
{
	master->forward_died(this, err);
}

tcp_forward *tcp_forward_listener::create_forward()
{
	return new tcp_forward;
}

void tcp_forward_listener::start_forwarding(tcp_forward_listener::forward_creator *fwdc, tcp_forward *fwd)
{
	if (DEBUG(CONNS))
		logger	<< timestamp << " forward [" << fwd->fileno(0) << ',' << fwd->fileno(1)
			<< "] connected" << endl;

	newconns.erase(fwdc);
	delete fwdc;

	connected(fwd);
}

void tcp_forward_listener::forward_died(tcp_forward_listener::forward_creator *fwdc, int err)
{
	int cfd = fwdc->fileno();
	int ofd = fwdc->sibling_fileno();

	if (DEBUG(CONNS))
		logger	<< timestamp << " connect [" << ofd << ',' << cfd << "] failed: " << errno_str(err) << endl;

	connect_died(ofd, cfd, err);
	newconns.erase(fwdc);
	delete fwdc;
	throw err;
}

void tcp_forward_listener::poll_newconn(int nfd)
{
	forward_creator *fwdc = new forward_creator(this, nfd);

	bool b = validate_connection(nfd, fwdc->fileno());
	if (DEBUG(CONNS)) {
		socket_address cliaddr;
		io_socket(nfd, false).get_peer_address(cliaddr);
		logger	<< timestamp << " new forward client [" << nfd << ',' << fwdc->fileno()
			<< "] on [" << fd << "] from " << cliaddr
			<< (b ? "" : " NOT") << " accepted" << endl;
	}
	if (b) {
		newconns.insert(fwdc);
		fwdc->connect(to);
	} else {
		delete fwdc;
	}
}

tcp_forward_listener::~tcp_forward_listener()
{
	for(newconn_set::iterator p = newconns.begin(); p != newconns.end(); ++p)
		delete *p;
}

void tcp_proxied_connection::connection_error(enum tcp_forward::forward_dir, int err)
{
	delete this;
	throw err;
}

void tcp_proxied_connection::slave_read_callback(tcp_forward::forwarding_side *_slave)
{
	if (_slave == slave[0])
		read_from_client();
	else	read_from_server();
}

void tcp_proxied_connection::slave_error_callback(tcp_forward::forwarding_side *_slave, int err)
{
	connection_error(_slave == slave[0] ? TO_CLIENT : TO_SERVER, err);
}

