#ifndef UDPMX_HH
#define UDPMX_HH

#include "ioclass.hh"
#include "packetio.hh"
#include <ext/hash_map>

class udp_mx_peer
{
	friend class udp_mx;
protected:
	udp_mx *mx;
public:
	socket_address remote;
protected:
	virtual void pkt_read() {}
public:
	udp_mx_peer(udp_mx *_mx, const socket_address& _remote)
		: mx(_mx), remote(_remote) {}
	virtual ~udp_mx_peer() {}

	void pkt_send(const virtual_buffer<char>& pkt);
	void drop();
};

namespace __gnu_cxx {
	template <>
	struct hash<socket_address::ipportpair>
	{
		size_t operator()(const socket_address::ipportpair& ipp) const
		{
			return (ipp.ip & 0xFFFF)|(ipp.ip >> 16)|ipp.port;
		}
	};
};

class udp_mx : public addressed_packet_io
{
protected:
	typedef __gnu_cxx::hash_map<socket_address::ipportpair, udp_mx_peer *> peermap;

	bool running;
	peermap peers;

	virtual udp_mx_peer *create_peer() { return new udp_mx_peer(this, packet_from); }
	virtual bool validate_new_peer() { return true; }
	void pkt_read();
	void poll_error(int) {}
public:
	udp_mx();
	~udp_mx();

	void start();
	void add_peer(udp_mx_peer *peer);
	void drop_peer(udp_mx_peer *peer);
	udp_mx_peer *find_peer(const socket_address::ipportpair& addr);
};

inline void udp_mx_peer::pkt_send(const virtual_buffer<char>& pkt)
{
	mx->out_add_to(pkt, remote);
}

inline void udp_mx_peer::drop()
{
	mx->drop_peer(this);
}

#endif /* !UDPMX_HH */
